//
//  Ship.swift
//  GameBattleshipTerminal
//
//  Created by Piotr Fulmański on 05/10/2019.
//  Copyright © 2019 Piotr Fulmański. All rights reserved.
//

import Foundation

class Ship {
    private let size: Int
    private var readyLevel: Int
    private var position: [(row: Int, col: Int, status: Status)]
//    private var status = Status.ready
    
    var isDestroyed: Bool {
        return readyLevel == 0 ? true : false
    }
    
    enum Status {
        case damaged, ready, destroyed
    }
    
    enum Direction {
        case up, down, left, right
    }
    
    init(size: Int, position: [(row: Int, col: Int, status: Status)]) {
        self.size = size
        self.position = position
        readyLevel = position.count
    }
    
    func autoMarkWhenDestroyed(onBoard: Board) {
        let allCellsArround = [(rowModifier: -1, colModifier:  0),// top
            (rowModifier: -1, colModifier: +1),// top-right
            (rowModifier:  0, colModifier: +1),// right
            (rowModifier: +1, colModifier: +1),// bottom-right
            (rowModifier: +1, colModifier:  0),// bottom
            (rowModifier: +1, colModifier: -1),// bottom-left
            (rowModifier:  0, colModifier: -1),// left
            (rowModifier: -1, colModifier: -1)// top-left
        ]
        
        var row, col: Int
        
        for (r, c, _) in position {
            for modifier in allCellsArround {
                row = r+modifier.rowModifier
                col = c+modifier.colModifier
                
                if onBoard.board[row][col] == Board.CellContentType.empty ||
                    onBoard.board[row][col] == Board.CellContentType.shot ||
                    onBoard.board[row][col] == Board.CellContentType.notAllowed {
                        onBoard.board[row][col] = Board.CellContentType.rescue
                }
            }
        }
    }
    
//    func checkIfDestryed() -> Bool {
//        for (_, _, status) in position {
//            if status != Status.destroyed {
//                return false
//            }
//        }
//
//        return true
//    }
    
    func hitAt(row: Int, col: Int) {
        // enumerated is needed to avoid error:
        // Connot assign to property: 'coordinate' is a 'let' constant
        for (index, coordinate) in position.enumerated() {
            if coordinate.row == row && coordinate.col == col {
                position[index].status = Status.damaged
                readyLevel -= 1
                break
            }
        }
    }
    
    func isLocatedAt(row: Int, col: Int) -> Bool {
        for coordinate in position {
            if coordinate.row == row && coordinate.col == col {
                return true
            }
        }
        return false
    }
}
